#
# This game class is used to evaluate different strategies
# for the two players.
# It uses the hand iterators class.

require 'pp'
require_relative 'hand_iterators'

class Game

  def initialize(strategy)
    @strategy = strategy
  end


  def total_value(outer_count, inner_count)

    running_total = 0
    outer_iter = Double_hand_cycle_iterator.new(1.0, outer_count)

    loop do    # outer
      start = outer_iter.another
      break if start.nil?

      inner_iter = Double_hand_cycle_iterator.new(@strategy.r_value, inner_count)
      loop do     # inner
        add = inner_iter.another
        break if add.nil?

        one_outcome = play_single_hand(start, add)
        running_total += one_outcome
      end    # inner
    end    #outer

    overall_game_value = running_total.to_f / (outer_count**2 * inner_count**2)

    return overall_game_value

  end  # total_value


  #
  # Make a list of outcomes for one player using different
  # pure strategies. P1 chooses bet or check.
  # P2 chooses call or drop.
  #
  def compare_choices(player_number, outer_count, inner_count)

    results = []

    #outer_iter = Single_hand_random_iterator.new(1.0, outer_count)
    outer_iter = Single_hand_cycle_iterator.new(1.0, outer_count)

    loop do   # outer

      bet_total = 0       # used by player 1 only
      check_total = 0     # used by player 1 only

      call_total = 0      # used by player 2 only
                          # Note: drop_value = 0 by definition

      outer_hand = outer_iter.another
      break if outer_hand.nil?

      #inner_iter = Single_hand_random_iterator.new(1.0, inner_count)
      inner_iter = Single_hand_cycle_iterator.new(1.0, inner_count)

      loop do   # inner

        inner_hand = inner_iter.another
        break if inner_hand.nil?

        #add_iter = Double_hand_random_iterator.new(@strategy.r_value, inner_count)
        add_iter = Double_hand_cycle_iterator.new(@strategy.r_value, inner_count)

        loop do   # adds

          adds = add_iter.another
          break if adds.nil?

          if player_number == 1
            bet, check = p1_play_two_ways(outer_hand, inner_hand, adds)
            bet_total += bet
            check_total += check
          else
            call = p2_play_one_way(inner_hand, outer_hand, adds)
            call_total += call
          end

        end    # adds

      end  # inner

      if player_number == 1
        bet_value = bet_total.to_f / inner_count**3
        check_value = check_total.to_f / inner_count**3
        results << [outer_hand, bet_value, check_value]
      else
        call_value = call_total.to_f / inner_count**3
        results << [outer_hand, call_value]
      end

    end   # outer


    return results

  end

end


# work out the implications of one particular hand
# assume beta (bet size) = 1 unit
#

def play_single_hand(start, add)
  p1_start = start[0]
  p2_start = start[1]

  if p1_start < @strategy.b_bet or p1_start > @strategy.d_bluff
    # p1 bets
    if p2_start < @strategy.c_call
      # p2 calls, p1 loses 1 or wins 2
      trial = p1_start + add[0] < p2_start + add[1] ? 2 : -1
    else
      # p2 drops, p1 wins 1
      trial = 1
    end

  else
    # p1 checks, p1 wins  0 or 1
    trial = p1_start + add[0] < p2_start + add[1] ? 1 : 0
  end

  return trial

end  # play_single_hand


def p1_play_two_ways(p1_hand, p2_hand, adds)

  # P1 bets
  if p2_hand < @strategy.c_call
    # p2 calls
    bet = p1_hand + adds[0] < p2_hand + adds[1] ? 2 : -1
  else
    bet = 1.0     # p2 drops
  end

  # P1 checks
  check = p1_hand + adds[0] < p2_hand + adds[1] ? 1 : 0

  return bet, check

end  # p1_play_two_ways


#
# Compare results for P2 if she calls (vs dropping, which is 0)
#
def p2_play_one_way(p1_hand, p2_hand, adds)

  if p1_hand < @strategy.b_bet or p1_hand > @strategy.d_bluff
    # p1_bets
    call = p1_hand + adds[0] < p2_hand + adds[1] ? -1 : 2
  else
    # we're only concerned about the difference between call and drop
    # if p1 doesn't bet, they are the same
    call = 0
  end

  return call

end